'use server';

import 'server-only';
import { encodedRedirect } from '@/utils/utils';
import { createServerClient } from '@/utils/supabase';
import type { User } from '@supabase/supabase-js';
import { headers } from 'next/headers';
import { redirect } from 'next/navigation';
import type { Result } from './index';

export const signUp = async (formData: FormData) => {
  const name = formData.get('name') as string;
  const email = formData.get('email') as string;
  const password = formData.get('password') as string;
  const supabase = await createServerClient();
  //const origin = (await headers()).get('origin');

  if (!email || !password) {
    return encodedRedirect(
      'error',
      '/sign-up',
      'Email & password are required',
    );
  }

  const { data, error } = await supabase.auth.signUp({
    email,
    password,
    //options: {
    //emailRedirectTo: `${origin}/auth/callback`,
    //},
  });

  if (error) {
    return redirect('/');
    //return encodedRedirect('error', '/sign-up',
    //'Thanks for signing up! Please check your email for a verification link.');
  } else {
    try {
      if (!data.user) throw new Error('Could not sign up');
      const { error } = await supabase
        .from('profiles')
        .update({
          full_name: name,
          provider: 'email',
        })
        .eq('id', data.user.id);
      if (error) throw new Error('Could not update profile');
    } catch (error) {
      console.error('Error updating profile: ', error);
    } finally {
      return redirect('/');
      //return encodedRedirect('success', '/',
      //'Thanks for signing up! Please check your email for a verification link.);
    }
  }
};

export const signIn = async (formData: FormData) => {
  const email = formData.get('email') as string;
  const password = formData.get('password') as string;
  const supabase = await createServerClient();

  const { error } = await supabase.auth.signInWithPassword({
    email,
    password,
  });

  if (error) {
    return encodedRedirect('error', '/sign-in', error.message);
  }
  return redirect('/');
};

export const forgotPassword = async (formData: FormData) => {
  const email = formData.get('email') as string;
  const supabase = await createServerClient();
  const origin = (await headers()).get('origin');
  const callbackUrl = formData.get('callbackUrl') as string;

  if (!email) {
    return encodedRedirect('error', '/forgot-password', 'Email is required');
  }

  const { error } = await supabase.auth.resetPasswordForEmail(email, {
    redirectTo: `${origin}/auth/callback?redirect_to=/reset-password`,
  });

  if (error) {
    return encodedRedirect(
      'error',
      '/forgot-password',
      'Could not reset password',
    );
  }

  if (callbackUrl) {
    return redirect(callbackUrl);
  }

  return encodedRedirect(
    'success',
    '/forgot-password',
    'Check your email for a link to reset your password.',
  );
};


export const resetPassword = async (
  formData: FormData | {password: string, confirmPassword: string}
) => {
  let password = '';
  let confirmPassword = '';

  if (formData instanceof FormData) {
    password = formData.get('password') as string;
    confirmPassword = formData.get('confirmPassword') as string;
  } else {
    password = formData.password;
    confirmPassword = formData.confirmPassword;
  }
  
  if (!password || !confirmPassword) {
    encodedRedirect(
      'error',
      '/reset-password',
      'Password and confirm password are required',
    );
  }

  const supabase = await createServerClient();

  if (password !== confirmPassword) {
    encodedRedirect('error', '/reset-password', 'Passwords do not match');
  }

  const { error } = await supabase.auth.updateUser({
    password: password,
  });

  if (error) {
    encodedRedirect('error', '/reset-password', 'Password update failed');
  }

  encodedRedirect('success', '/reset-password', 'Password updated');
};


export const resetPasswordFromEmail = async (formData: FormData) => {

   const password = formData.get('password') as string;
   const confirmPassword = formData.get('confirmPassword') as string;
  
  if (!password || !confirmPassword) {
    encodedRedirect(
      'error',
      '/reset-password',
      'Password and confirm password are required',
    );
  }

  const supabase = await createServerClient();

  if (password !== confirmPassword) {
    encodedRedirect('error', '/reset-password', 'Passwords do not match');
  }

  const { error } = await supabase.auth.updateUser({
    password: password,
  });

  if (error) {
    encodedRedirect('error', '/reset-password', 'Password update failed');
  }

  encodedRedirect('success', '/reset-password', 'Password updated');
};

export const signOut = async () => {
  const supabase = await createServerClient();
  await supabase.auth.signOut();
  return redirect('/sign-in');
};

export const getUser = async (): Promise<Result<User>> => {
  try {
    const supabase = await createServerClient();
    const { data, error } = await supabase.auth.getUser();
    if (error) throw error;
    return { success: true, data: data.user };
  } catch (error) {
    return { success: false, error: 'Could not get user!' };
  }
};
